# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os.path

from DatabaseLibrary import DatabaseLibrary
from SeleniumLibrary import SeleniumLibrary
from robot.libraries import DateTime
from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Collections import Collections
from robot.libraries.OperatingSystem import OperatingSystem
from robot.api import logger

__all__ = ('sl', 'bi', 'dtl', 'osl', 'cl', 'get_artifacts_dir')


def get_artifacts_dir(delta_path=""):
    output_path = bi().get_variable_value("${EXECDIR}")
    output_path += "/Artifacts/"
    output_path += delta_path
    output_path_normalized = validate_create_artifacts_dir(output_path)
    return output_path_normalized


def validate_create_artifacts_dir(path):
    """
    As input take path return normalized path,
    create directory for this path
    :param path:
    :return path_normalized:
    """
    output_dir_normalized = os.path.dirname(os.path.abspath(os.path.normpath(path)))
    output_path_normalized = os.path.abspath(os.path.normpath(path))
    if not os.path.exists(output_dir_normalized):
        os.makedirs(output_dir_normalized)
    return output_path_normalized


def s2l():
    """
        :rtype : SeleniumLibrary
        """
    s2l_instance = BuiltIn().get_library_instance('SeleniumLibrary')
    assert isinstance(s2l_instance, SeleniumLibrary)
    return s2l_instance


def sl():
    """
        :rtype : SeleniumLibrary
        """
    sl_instance = BuiltIn().get_library_instance('SeleniumLibrary')
    assert isinstance(sl_instance, SeleniumLibrary)
    return sl_instance


def bi():
    """
        :rtype : BuiltIn
        """
    bi_instance = BuiltIn().get_library_instance('BuiltIn')
    assert isinstance(bi_instance, BuiltIn)
    return bi_instance


def dtl():
    """

        :rtype : DateTime
        """
    dt_instance = BuiltIn().get_library_instance('DateTime')
    return dt_instance


def osl():
    """

        :rtype : OperatingSystem
        """
    os_instance = BuiltIn().get_library_instance('OperatingSystem')
    assert isinstance(os_instance, OperatingSystem)
    return os_instance


def cl():
    """

        :rtype : Collections
        """
    c_instance = BuiltIn().get_library_instance('Collections')
    assert isinstance(c_instance, Collections)
    return c_instance


def dbl():
    """

        :rtype : DatabaseLibrary
        """
    c_instance = BuiltIn().get_library_instance('DatabaseLibrary')
    assert isinstance(c_instance, DatabaseLibrary)
    return c_instance


def ttmkl():
    """

        :rtype : TestToolsMK
        """
    c_instance = BuiltIn().get_library_instance('TestToolsMK')
    # assert isinstance(c_instance, TestToolsMK)
    return c_instance



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os
import os.path
import time

from robot.api import logger
from robot.libraries import DateTime
from selenium.webdriver import FirefoxProfile, ChromeOptions
from selenium.webdriver.common.keys import Keys

from TestToolsMK.robot_instances import validate_create_artifacts_dir, sl, bi

try:
    # noinspection PyCompatibility
    from urlparse import urljoin
except ImportError:  # python3
    # noinspection PyCompatibility,PyUnresolvedReferences
    from urllib.parse import urljoin


# noinspection PyProtectedMember
class SeleniumLibraryKeywords(object):
    WIDTH_DEFAULT = "1366"
    HEIGHT_DEFAULT = "768"
    SELENIUM_SPEED = "0 sec"
    SELENIUM_TEST_BROWSER = "ff"
    SELENIUM_TIMEOUT = "5 s"
    # noinspection PyPep8
    XPATH2_JS = 'if(!window.jQuery){var headID = document.getElementsByTagName("head")[0]; var newScript = document.createElement(\'script\'); newScript.type=\'text/javascript\'; newScript.src=\'http://llamalab.com/js/xpath/minified/XPath.js\'; headID.appendChild(newScript);}'
    # noinspection PyPep8
    JQUERY_JS = "if(!window.jQuery){var headID = document.getElementsByTagName(\"head\")[0]; var newScript = document.createElement('script'); newScript.type='text/javascript'; newScript.src='http://code.jquery.com/jquery-2.1.4.min.js'; headID.appendChild(newScript);}"

    @staticmethod
    def open_new_tab(url):
        """Hack it use Control +t to open new tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + 't')
        time.sleep(2)
        sl().go_to(url)

    @staticmethod
    def switch_tab_by_id(id_tab):
        """Hack it use Control + 1,2,3 etc to switch tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + id_tab)
        time.sleep(4)
        # actions = ActionChains(driver)
        # actions.key_down(Keys.CONTROL).key_down(Keys.TAB).key_up(Keys.TAB).key_up(Keys.CONTROL).perform()

    @staticmethod
    def press_key_python(command, locator="//body", strategy="XPATH"):
        """Hack !!!  example argument | Keys.CONTROL + 't' |Keys.TAB + Keys.SHIFT"""
        driver = sl().driver
        element = driver.find_element(eval("By." + strategy), locator)
        element.send_keys(eval(command))

    @staticmethod
    def close_tab():
        """Hack it use Control +w to close tab"""
        driver = sl().driver
        body = driver.find_element_by_tag_name("body")
        body.send_keys(Keys.CONTROL + 'w')

    @staticmethod
    def set_browser_size_and_position(width=WIDTH_DEFAULT, height=HEIGHT_DEFAULT, x=0, y=0):
        sl().set_window_size(width, height)
        sl().set_window_position(x, y)

    @staticmethod
    def go_to_smart(url):
        """Redirect only in on different url"""
        current_url = sl().get_location()
        if url != current_url:
            sl().go_to(url)

    @staticmethod
    def click_element_extended(locator, timeout=None, error_msg=None):
        """Click element proceed with following steps
        1.wait_until_page_contains_element
        2.wait_until_element_is_visible_wait_until_element_is_visible
        3.mouse_over"""
        sl().wait_until_page_contains_element(locator, timeout, error_msg)
        sl().wait_until_element_is_visible(locator, timeout, error_msg)
        sl().mouse_over(locator)
        sl().click_element(locator)

    @staticmethod
    def double_click_element_extended(locator, timeout=None, error=None):
        sl().wait_until_page_contains_element(locator, timeout, error)
        sl().wait_until_element_is_visible(locator, timeout, error)
        sl().mouse_over(locator)
        sl().double_click_element(locator)

    def click_element_extended_and_wait(self, locator, sleep, timeout=None, error_msg=None, reason=None):
        self.click_element_extended(locator, timeout, error_msg)
        bi().sleep(sleep, reason)

    @staticmethod
    def open_browser_extension(url, browser="ff", width=WIDTH_DEFAULT, height=HEIGHT_DEFAULT, x="0", y="0", alias=None, remote_url=False,
            desired_capabilities=None, ff_profile_dir=None, selenium_timeout=SELENIUM_TIMEOUT, keyword_to_run_on_failure="Capture Page Screenshot Extension"):
        sl().open_browser("about:blank", browser, alias, remote_url, desired_capabilities, ff_profile_dir)
        sl().set_window_position(x, y)
        sl().set_window_size(width, height)
        sl().set_selenium_timeout(selenium_timeout)
        sl().register_keyword_to_run_on_failure(keyword_to_run_on_failure)
        sl().go_to(url)

    def import_xpath2(self):
        sl().execute_javascript(self.XPATH2_JS)

    # noinspection PyPep8Naming,PyPep8Naming
    def import_jQuery(self):
        sl().execute_javascript(self.JQUERY_JS)

    # noinspection PyProtectedMember
    @staticmethod
    def capture_page_screenshot_extension(prefix="", postfix="", add_time_stamp=True, add_test_case_name=True, add_file_path_to_list="${list of screenshots}",
            output_dir="Artifacts/Screenshots"):
        output_dir_normalized = validate_create_artifacts_dir(output_dir)

        if add_time_stamp:
            current_time = " " + DateTime.get_current_date(result_format="%Y.%m.%d_%H.%M.%S")
        else:
            current_time = ""
        if add_test_case_name:
            test_case_name = bi().get_variable_value("${TEST_NAME}")
        else:
            test_case_name = ""

        output_file = output_dir_normalized + "/" + prefix + test_case_name + postfix + current_time + ".png"
        output_file_normalized = os.path.normpath(output_file)

        # sl()driver.get_screenshot_as_file(output_file_normalized)
        sl().capture_page_screenshot(output_file_normalized)

        results = bi().run_keyword_and_return_status("Variable Should Exist", add_file_path_to_list)

        if not results:
            bi()._get_var_name(add_file_path_to_list)
            list_with_files = bi().create_list(output_file_normalized)
            bi().set_test_variable(add_file_path_to_list, list_with_files)
        else:
            list_with_files = bi().create_list(output_file_normalized)
            list_with_files = bi().run_keyword("Combine Lists", add_file_path_to_list, list_with_files)
            bi().set_test_variable(add_file_path_to_list, list_with_files)

        return output_file_normalized

    @staticmethod
    def element_attribute_should_be(locator, attribute, attribute_value_expected, msg=None, values=True):
        actual_value = sl().get_element_attribute(locator + "@" + attribute)
        # noinspection PyProtectedMember
        actual_value, attribute_value_expected = [bi()._convert_to_string(i) for i in (actual_value, attribute_value_expected)]
        bi()._should_be_equal(actual_value, attribute_value_expected, msg, values)

    # noinspection SpellCheckingInspection,SpellCheckingInspection,SpellCheckingInspection,SpellCheckingInspection
    @staticmethod
    def create_download_dir_profile_for_firefox(path_to_download, mime_types_file=None, *extensions_files):
        """
        Example use
        | ${profile} | Create Download Dir Profile For Firefox | Artifacts | Resources/mimeTypes.rdf | Resources/webdriver_element_locator-2.0-fx.xpi | Resources/selenium_ide-2.9.1-fx.xpi |
        | Open Browser Extension | https://support.spatialkey.com/spatialkey-sample-csv-data/ | ff_profile_dir=${profile} |
        | Click Element | //a[contains(@href,'sample.csv.zip')]  |
        """
        path_to_download_check = validate_create_artifacts_dir(path_to_download)

        fp = FirefoxProfile()
        fp.set_preference("browser.download.folderList", 2)
        fp.set_preference("browser.download.manager.showWhenStarting", False)
        fp.set_preference("browser.download.manager.alertOnEXEOpen", False)
        fp.set_preference("browser.download.dir", path_to_download_check)
        fp.set_preference("xpinstall.signatures.required", False)
        fp.set_preference("browser.helperApps.alwaysAsk.force", False)
        fp.set_preference("browser.helperApps.neverAsk.saveToDisk",
            "application/msword;application/csv;text/csv;image/png;image/jpeg;application/pdf;text/html;text/plain;application/octet-stream")
        fp.set_preference("pdfjs.disabled", True)
        fp.update_preferences()
        for single_extension in extensions_files:
            fp.add_extension(single_extension)
        if mime_types_file is not None:
            mime_types_file = os.path.abspath(mime_types_file)
            from shutil import copy2
            copy2(os.path.normpath(mime_types_file), fp.profile_dir)
        logger.info("Firefox Profile Created in dir '" + fp.profile_dir + "'")
        return fp.profile_dir

    # noinspection SpellCheckingInspection,SpellCheckingInspection
    @staticmethod
    def create_download_dir_capabilities_for_chrome(path_to_download, **extensions_files):
        """
        Example use
        | ${capabilities} |	create_download_dir_capabilities_for_chrome	| Artifacts |
        | Open Browser Extension | https://support.spatialkey.com/spatialkey-sample-csv-data/ |	gc | desired_capabilities=${capabilities} |
        | Click Element	 | //a[contains(@href,'sample.csv.zip')] |
        """

        path_to_download_check = validate_create_artifacts_dir(path_to_download)

        chrome_options = ChromeOptions()
        prefs = {"download.default_directory": path_to_download_check, "directory_upgrade": "true"}

        chrome_options.add_experimental_option("prefs", prefs)
        chrome_options.add_argument("--disable-web-security")
        for single_extension in extensions_files:
            chrome_options.add_extension(single_extension)

        logger.info("Chrome Capabilities set download dir '" + path_to_download_check + "'")
        return chrome_options.to_capabilities()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import json
import os
import os.path
import platform
import shlex
import subprocess
import urllib

import requests

from TestToolsMK.robot_instances import osl
import time

try:
    # noinspection PyCompatibility
    from urlparse import urljoin
except ImportError:  # python3
    # noinspection PyCompatibility,PyUnresolvedReferences
    from urllib.parse import urljoin

from robot.api import logger


def sizeof_fmt(num, suffix='B'):
    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)


def wait_net_service(server, port, timeout=None):
    """ Wait for network service to appear
        @param port:
        @param server:
        @param timeout: in seconds, if None or 0 wait forever
        @return: True of False, if timeout is None may return only True or
                 throw unhandled network exception
    """
    # noinspection PyGlobalUndefined,PyGlobalUndefined
    global end, now
    import socket
    import errno

    s = socket.socket()
    if timeout:
        from time import time as now
        # time module is needed to calc timeout shared between two exceptions
        end = now() + timeout

    while True:
        try:
            if timeout:
                next_timeout = end - now()
                if next_timeout < 0:
                    return False
                else:
                    s.settimeout(next_timeout)

            s.connect((server, port))

        except socket.timeout as err:
            # this exception occurs only if timeout is set
            if timeout:
                return False

        except socket.error as err:
            # catch timeout exception from underlying network library
            # this one is different from socket.timeout
            if type(err.args) != tuple or err[0] != errno.ETIMEDOUT:
                raise

        except:
            # catch timeout exception from underlying network library
            # this one is different from socket.timeout
            if timeout:
                return False
        else:
            s.close()
            return True


class UtilsKeywords(object):
    @property
    def get_latest_chrome_driver_version(self):
        return urllib.urlopen('http://chromedriver.storage.googleapis.com/LATEST_RELEASE').read().strip()

    @property
    def get_latest_firefox_driver_version(self):
        json_raw = urllib.urlopen('https://api.github.com/repos/mozilla/geckodriver/releases/latest').read().strip()
        json_data = json.loads(json_raw)
        return json_data["tag_name"].strip()

    @property
    def get_url_for_latest_chrome_driver(self):
        last_build = self.get_latest_chrome_driver_version
        try:
            base_url_for_driver = "http://chromedriver.storage.googleapis.com/" + last_build + "/"
            if platform.system() == "Windows":
                return base_url_for_driver + "chromedriver_win32.zip"
            if platform.system() == "Linux":
                return base_url_for_driver + "chromedriver_linux64.zip"

        except os.error as e:
            logger.warn("Unexpected error" + e)
            return "missing"

    @property
    def get_url_for_latest_firefox_driver(self):
        last_build = self.get_latest_firefox_driver_version
        try:
            base_url_for_driver = "https://github.com/mozilla/geckodriver/releases/download/" + last_build + "/"
            if platform.system() == "Windows":
                return base_url_for_driver + "geckodriver-" + last_build + "-win64.zip"
            if platform.system() == "Linux":
                return base_url_for_driver + "geckodriver-" + last_build + "-linux64.tar.gz"

        except os.error as e:
            logger.warn("Unexpected error" + e)
            return "missing"


    def get_chrome_driver_latest(self, path='./bin'):
        """
        Download Latest Chrome Driver and add it to system path (only for this session).
        If system already contains proper version do nothing.
        Based on information from http://chromedriver.storage.googleapis.com/LATEST_RELEASE
        """
        try:
            initial = osl().get_environment_variable("PATH")
            path_abstract = os.path.abspath(path)

            if path_abstract not in initial:
                osl().set_environment_variable("PATH", path_abstract + os.pathsep + initial)

            try:
                version = subprocess.check_output(["chromedriver", "--version"], shell=True, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                version = "Chrome driver is MISSING "

            version_current = version.strip()
            version_latest = self.get_latest_chrome_driver_version
            url_chrome = self.get_url_for_latest_chrome_driver
            logger.debug("Current version \t" + version_current)
            logger.debug("Latest version :\t" + version_latest)
            if version_latest not in version_current:
                logger.info("Current version %s , latest is %s" % (version_current, version_latest))
                if not os.path.exists(path_abstract):
                    os.makedirs(path_abstract)
                logger.info("start download chrome driver :" + url_chrome)

                driver = urllib.urlopen(url_chrome).read()
                logger.debug("Chrome driver compressed size: " + sizeof_fmt(len(driver)))

                from zipfile import ZipFile
                from io import BytesIO
                with ZipFile(BytesIO(driver)) as zfile:
                    zfile.extractall(path_abstract)
                logger.info("Chrome driver extracted to : " + path_abstract)

            else:
                logger.info("Latest Version of ChromeDriver Present %s , latest %s" % (version_current, version_latest))

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    def get_firefox_driver_latest(self, path='./bin'):
        """
        Download Latest Firefox geckodriver Driver and add it to system path (only for this session).
        If system already contains proper version do nothing.
        Based on information from https://api.github.com/repos/mozilla/geckodriver/releases/latest
        """

        try:
            initial = osl().get_environment_variable("PATH")
            path_abstract = os.path.abspath(path)

            if path_abstract not in initial:
                osl().set_environment_variable("PATH", path_abstract + os.pathsep + initial)

            try:
                version = subprocess.check_output(["geckodriver", "--version"], shell=True, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                version = "Firefox geckodriver driver is MISSING "

            version_current = version.strip()
            version_latest = self.get_latest_firefox_driver_version.replace("v", "").strip()
            url_firefox = self.get_url_for_latest_firefox_driver
            logger.debug("Current version \t" + version_current)
            logger.debug("Latest version :\t" + version_latest)
            if version_latest not in version_current:
                logger.info("Current version %s , latest is %s" % (version_current, version_latest))

                logger.info("start download firefox geckodriver driver :" + url_firefox)
                if not os.path.exists(path_abstract):
                    os.makedirs(path_abstract)
                driver = urllib.urlopen(url_firefox).read()
                logger.debug("Firefox geckodriver driver compressed size: " + sizeof_fmt(len(driver)))

                from zipfile import ZipFile
                from io import BytesIO
                with ZipFile(BytesIO(driver)) as zfile:
                    zfile.extractall(path_abstract)
                logger.info("Chrome driver extracted to : " + path_abstract)

            else:
                logger.info("Latest Version of Firefox geckodriver Present %s , latest %s" % (version_current, version_latest))

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    @staticmethod
    def get_selenium_server(url='https://goo.gl/lbAQcq', path='./bin/selenium-server.jar',
                            skip_if_already_exists="True"):
        """
        Currently hard coded as arg future change to download from https://selenium-release.storage.googleapis.com
        """
        dir_selenium = os.path.dirname(path)
        path = os.path.abspath(path)
        logger.debug("Folder to download " + dir_selenium)

        if not os.path.exists(dir_selenium):
            os.makedirs(dir_selenium)

        if skip_if_already_exists == "True":
            if os.path.exists(path):
                logger.info(
                    "Skip Selenium Server download already exists in path \"%s\" because skip_if_already_exists is set to \"%s\"" % (
                    path, skip_if_already_exists))
                return

        try:
            r = requests.head(url, allow_redirects=True)
            logger.info("Resolved url: \t" + r.url)
            testfile = urllib.URLopener()
            testfile.retrieve(r.url, path)
            logger.info("Selenium Server download completed to " + path)

        except KeyError as e:
            logger.error(e)
        except OSError as e:
            logger.error(e)

    def start_selenium_server(self, path='./bin/selenium-server.jar', timeout=60, port="4444", logs_path='./bin'):
        """
        :return:
        """
        path = os.path.abspath(path)

        with open(os.path.abspath(logs_path + "/selenium_server_stdout.txt"), "wb") as out, open(os.path.abspath(logs_path + "/selenium_server_stderr.txt"),
                "wb") as err:
            command = "java -jar \"" + path + "\" -port " + port
            logger.info("Command to start server :" + command)
            # noinspection PyAttributeOutsideInit
            self.selenium_server = subprocess.Popen(shlex.split(command), stdout=out, stderr=err)
        time.sleep(7)

    def shutdown_selenium_server(self):
        if self.selenium_server is not None :
            self.selenium_server.terminate()
            logger.info("Selenium Server shutdown")
        else:
            logger.error("Server not started")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski

from __future__ import print_function

import json
import re

import gspread
from robot.api import logger
from oauth2client.client import flow_from_clientsecrets
from oauth2client import client
from oauth2client.service_account import ServiceAccountCredentials
import gspread


class GoogleSheetsKeywords(object):
    SPREADSHEET = None  # type: gspread.Spreadsheet
    WORKSHEET = None  # type: gspread.Worksheet
    JSON_KEY = None  # type: file

    def __init__(self, key_json_file=None, google_document_id=None, worksheet_name=None):
        if key_json_file is not None:
            if google_document_id is not None:
                self.get_spreadsheet_by_id(key_json_file, google_document_id, worksheet_name)

    def get_spreadsheet_by_id(self, file_name, google_document_id, worksheet_name=None):
        self.JSON_KEY = json.load(open(file_name))
        scope = ['https://spreadsheets.google.com/feeds']
        credentials = ServiceAccountCredentials.from_json_keyfile_name(file_name, scope)
        gc = gspread.authorize(credentials)
        self.SPREADSHEET = gc.open_by_key(google_document_id)
        self.WORKSHEET = self.SPREADSHEET.sheet1
        if worksheet_name is not None:
            self.WORKSHEET = self.SPREADSHEET.worksheet(worksheet_name)
        else:
            self.WORKSHEET = self.SPREADSHEET.sheet1

        logger.info("Spreadsheet id opened '" + google_document_id + "'")
        logger.info("worksheet opened '" + self.WORKSHEET.title + "'")

    def select_worksheet_by_name(self, worksheet_name):
        self.WORKSHEET = self.SPREADSHEET.worksheet(worksheet_name)

    def get_dictionary_logins_and_passwords(self):
        return dict(zip(self.WORKSHEET.col_values(1), self.WORKSHEET.col_values(2)))

    def get_password_for_login(self, login):
        """Return password for provided login, rise error when login is missing"""
        dictionary = self.get_dictionary_logins_and_passwords()
        return dictionary[login]

    def find_cell_using_regex(self, regex):
        """Return password for provided login, rise error when login is missing"""
        pattern = r'%s' % regex
        print(pattern)
        amount_re = re.compile(pattern)
        return self.WORKSHEET.find(amount_re)

    def find_all_cell_using_regex(self, regex):
        """Return password for provided login, rise error when login is missing"""
        pattern = r'%s' % regex
        amount_re = re.compile(pattern)
        return self.WORKSHEET.findall(amount_re)

    def insert_row(self, values, index=1):
        """
        first argument is list, second optional index
        """
        self.WORKSHEET.insert_row(values, int(index))
        logger.info("Row inserted to index " + str(index) + " to Spreadsheet " + self.SPREADSHEET.title)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os
import time
import sys

import csv
# import unicodecsv as csv
from robot.libraries import DateTime
from robot.utils import asserts

from TestToolsMK import robot_instances
from TestToolsMK.robot_instances import validate_create_artifacts_dir


class CsvKeywords(object):
    OUTPUT_FILE_CSV = "Artifacts/output.csv"

    def csv_set_output_file(self, file_name="Artifacts/output.csv"):
        self.OUTPUT_FILE_CSV = validate_create_artifacts_dir(file_name)

    @staticmethod
    def append_to_csv(filename, values_list, encoding='UTF-8'):
        """
        Example usage:
        | ${list} | Create List	| a | ""1"" |	"é,őáá" | #example with chars utf-8 |
        | Append To Csv | example.csv   |
        """
        if sys.version_info[0] < 3:
            with open(filename, 'ab') as csv_file:
                writer_csv = csv.writer(csv_file, dialect='excel')
                writer_csv.writerow([item.encode(encoding) for item in values_list])
        else:
            with open(filename, 'a', newline='') as csv_file:
                writer_csv = csv.writer(csv_file, dialect='excel')
                writer_csv.writerow([item for item in values_list])

    def csv_writer(self, *values):
        """
        Store to default file records in csv
        ${EXECDIR}/Artifacts/output.csv
        change file name using csv change output file
        """
        log_file = self.OUTPUT_FILE_CSV
        self.append_to_csv(log_file, list(values))

    def csv_writer_with_extra(self, *values):
        """
        Add extra params at beginning
        1. time of execution
        2. suite + test cases name
        """
        test_case_name = str(robot_instances.bi().get_variable_value("${TEST_NAME}"))
        suite_name = str(robot_instances.bi().get_variable_value("${SUITE_NAME}"))
        extra_list = list(values)
        extra_list.insert(0, suite_name + test_case_name)
        self.csv_writer_with_time(*extra_list)

    def csv_writer_with_time(self, *values):
        current_time = DateTime.get_current_date(result_format="%Y.%m.%d %H:%M:%S")
        extra_list = list(values)
        extra_list.insert(0, current_time)
        self.csv_writer(*extra_list)

    @staticmethod
    def file_should_not_change(filename, time_in_sec="1", msg="File was modify during waiting time"):
        """
        Methods check modification date date if date doesnt change after set time return true
        Best use with method Wait Until Keyword Succeeds
        """
        before = os.stat(filename).st_mtime
        time.sleep(float(time_in_sec))
        after = os.stat(filename).st_mtime
        asserts.assert_equal(before, after, msg, values=False)

    @staticmethod
    def append_to_file_at_beginning(path, content, encoding="UTF-8"):
        path = os.path.abspath(path)
        parent = os.path.dirname(path)
        if not os.path.exists(parent):
            os.makedirs(parent)
        if not os.path.isfile(path):
            open(path, 'w').close()
        with open(path, 'r') as original:
            data = original.read()
        final_content = content + "\n" + data
        with open(path, 'w') as modified:
            modified.write(final_content.encode(encoding))
        # noinspection PyProtectedMember
        robot_instances.osl()._link("Appended to file begin of file '%s'.", path)

    @staticmethod
    def get_file_lines_count(path):
        with open(path) as f:
            for i, l in enumerate(f):
                pass
            # noinspection PyUnboundLocalVariable
            return i + 1

    @staticmethod
    def csv_read_file(path, encoding='UTF-8', encoding_errors='strict'):
        """
        returns file CSV content as 2D table
        """
        output_table = []
        # encoding = osl()._map_encoding(encoding)
        with open(path) as csv_file:
            csv_reader = csv.reader(csv_file, quotechar='"')
            for row in csv_reader:
                output_table.append(row)
            return output_table



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import os
# from xmlrpclib import DateTime
import string
import random
from datetime import datetime

from robot.api import logger
from robot.libraries.DateTime import Date
from robot.libraries.DateTime import Time
from robot.utils import (is_falsy)

from TestToolsMK.robot_instances import dbl, ttmkl, validate_create_artifacts_dir


def get_current_time_for_timers():
    return datetime.now()


# noinspection PyProtectedMember
class SQLKeywords(object):
    OUTPUT_FILE_LOG_SQL = "Artifacts/log_of_sql_execution.sql"
    ADD_LOGS_FLAG = True

    def set_sql_log_output_file(self, path="Artifacts/log_of_sql_execution.sql"):
        self.OUTPUT_FILE_LOG_SQL = validate_create_artifacts_dir(path)

    def set_add_logs_flag(self, flag=False):
        self.ADD_LOGS_FLAG = flag

    def query_many_rows(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :param select_statement:
        :param append_to_logs:
        :return:
        """
        if append_to_logs:
            self._add_query_to_log_file(select_statement)
        results = dbl().query(select_statement)
        if append_to_logs:
            self._add_results_to_log_file(results)
        return results

    def query_row(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :raise error when results contains more then one row results
        :return: table
        """
        results = self.query_many_rows(select_statement, append_to_logs)

        if len(results) > 1:
            message = "Error. Results contains more then one row. Actual size is %s " % (len(results))
            self._append_to_file("/* \n" + message + "\n */")
            raise AssertionError(message)

        return results[0]

    def query_cell(self, select_statement, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :raise error when results contains more then one cell
        :return: single value
        """
        results = self.query_row(select_statement, append_to_logs)
        if len(results) > 1:
            message = "Error. Results contains more then one cell. Actual size is %s " % (len(results))
            self._append_to_file("/* \n" + message + "\n */")
            raise AssertionError(message)
        return results[0]

    def _add_query_to_log_file(self, statement):
        current_time = get_current_time_for_timers()
        self.start_time = current_time
        final_string = "\n/* Start execution, statement below : " + current_time.strftime(
            "%Y.%m.%d %H:%M:%S") + " */\n" + statement + "\n"
        self._append_to_file(final_string)

    def _add_results_to_log_file(self, results):
        current_time = get_current_time_for_timers()
        total_time_verbose = Time(current_time - self.start_time).convert("verbose")

        if results:
            data = "[\n"
            for i in range(len(results)):
                data += "("
                for j in range(len(results[i])):
                    data += str(results[i][j])
                    if j != (len(results[i]) - 1):
                        data += "\t,"
                data += ")\n"
            data += "]"
        else:
            data = ""
        final_string = "/* Response of statement in :" + total_time_verbose + " , data below " + data + " */\n"
        self._append_to_file(final_string)

    def _append_to_file(self, text):
        full_log_file_path = validate_create_artifacts_dir(self.OUTPUT_FILE_LOG_SQL)
        mode = 'a' if os.path.exists(full_log_file_path) else 'w'
        with open(full_log_file_path, mode) as output:
            output.write(text)

    def execute_sql_string_with_logs(self, sql_string, append_to_logs=ADD_LOGS_FLAG):
        """
        To switch output file with logs use
        | Set Sql Log Output File | ./myFile.sql |
        :param sql_string:
        :param append_to_logs:
        :return:
        """
        if append_to_logs:
            self._add_query_to_log_file(sql_string)
        dbl().execute_sql_string(sql_string)
        if append_to_logs:
            self._add_results_to_log_file(None)

    def insert_data_to_table(self, table_name, data):
        """
        return table name, table will have columns with names 'c0', 'c1' for all columns
        table will be also reindex after insert
        """
        cur = dbl()._dbconnection.cursor()

        size = len(data)
        if size < 1:
            raise AssertionError("missing data 0 rows")

        row_size = len(data[0])
        if row_size < 1:
            raise AssertionError("missing data 0 columns")

        columns_desc = ""
        for index in range(len(data[0])):
            columns_desc += """"c""" + str(index) + """" VARCHAR"""
            if index < (row_size - 1):
                columns_desc += ","

        create_sql = "CREATE TABLE \"%s\" (%s)" % (table_name, columns_desc)

        logger.info(create_sql)
        cur.execute(create_sql)

        columns_names = ""
        values = ""
        for index in range(len(data[0])):
            columns_names += "c" + str(index)
            values += """?"""
            if index < (row_size - 1):
                columns_names += ", "
                values += ", "

        insert_data_sql = "INSERT INTO \"main\".\"%s\" (%s) VALUES (%s) " % (table_name, columns_names, values)

        logger.info(insert_data_sql)
        cur.executemany(insert_data_sql, data)

        reindex_sql = "REINDEX \"main\".\"%s\"" % table_name
        cur.execute(reindex_sql)

        dbl()._dbconnection.commit()

        return

    def insert_data_to_generated_table(self, data):
        table_name = table_name_generator()
        self.insert_data_to_table(table_name, data)
        return table_name

    def csv_read_file_to_database(self, table_name, csv_file, encoding='UTF-8', encoding_errors='strict'):
        """ Use only for temporary use
        connect to database in memory if not connected, create table with name and import data from csv file"""

        if dbl()._dbconnection is None:
            dbl().connect_to_database_using_custom_params("sqlite3", "':memory:'")
        array_table = ttmkl().csv_read_file(csv_file, encoding, encoding_errors)
        self.insert_data_to_table(table_name, array_table)
        return table_name

    def connect_to_database_using_jdbc_driver(self, jdbc_connection_string, user, password, jdbc_driver, jdbc_jar_path,
                                              library="jaydebeapi"):
        """
        Example parameters for CIS databases Cisto Integration Services, using java drivers is done using library
        jdbc_connection_string: jdbc:compositesw:dbapi@localhost:9401?domain=composite&dataSource=Example
        jdbc_driver: cs.jdbc.driver.CompositeDriver
        jdbc_jar_path: Binaries/csjdbc.jar
        jaydebeapi library has to be installed 

        """
        dbl().connect_to_database_using_custom_params(library, "\"%s\",\"%s\",[ \"%s\", \"%s\"],\"%s\"," % (
            jdbc_driver, jdbc_connection_string, user, password,
            jdbc_jar_path))


def table_name_generator(size=12, chars=string.ascii_lowercase + string.ascii_uppercase):
    return ''.join(random.choice(chars) for _ in range(size))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
from __future__ import print_function

import os

from TestToolsMK.collections_keywords import CollectionsKeywords
from TestToolsMK.csv_keywords import CsvKeywords
from TestToolsMK.excel_keywords import ExcelKeywords
from TestToolsMK.google_sheets_keywords import GoogleSheetsKeywords
from TestToolsMK.image_magick_keywords import ImageMagickKeywords
from TestToolsMK.logger_extension_keywords import LoggerKeywords
from TestToolsMK.pdf_keywords import PDFKeywords
from TestToolsMK.selenium_extentions_keywords import SeleniumLibraryKeywords
from TestToolsMK.send_android_notification_keywords import SendNotificationKeywords
from TestToolsMK.sqlkeywords import SQLKeywords
from TestToolsMK.timers_keywords import TimerKeywords
from TestToolsMK.utils import UtilsKeywords

__version_file_path__ = os.path.join(os.path.dirname(__file__), 'VERSION')
__version__ = open(__version_file_path__, 'r').read().strip()


class TestToolsMK(GoogleSheetsKeywords, SeleniumLibraryKeywords, TimerKeywords, CollectionsKeywords, ImageMagickKeywords, LoggerKeywords, CsvKeywords,
                  SQLKeywords, UtilsKeywords, SendNotificationKeywords, ExcelKeywords,PDFKeywords):
    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
    ROBOT_LIBRARY_VERSION = __version__




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski

from datetime import datetime

from robot.api import logger
from robot.libraries.DateTime import Date
from robot.libraries.DateTime import Time
from robot.utils import (is_falsy)


def get_current_time_for_timers():
    return datetime.now()


class TimerKeywords(Time, Date):
    TIMERS_DICTIONARY = {}

    def timer_start(self, timer_name="Global"):
        current_time = get_current_time_for_timers()
        logger.info(current_time)
        if timer_name in self.TIMERS_DICTIONARY:
            error = "Timer with name \"%s\" already stated at %s" % (timer_name, self.TIMERS_DICTIONARY[timer_name].__str__())
            logger.warn(error)
        self.TIMERS_DICTIONARY[timer_name] = current_time
        msg = "Timer with name \"%s\" stated at %s" % (timer_name, self.TIMERS_DICTIONARY[timer_name].__str__())
        logger.info(msg)

    def timer_stop(self, timer_name="Global", result_format="number", exclude_millis="True"):
        current_time = get_current_time_for_timers()
        if timer_name not in self.TIMERS_DICTIONARY:
            message = "Time '%s' is not started." % timer_name
            raise AssertionError(message)
        else:
            delta = Time(current_time - self.TIMERS_DICTIONARY[timer_name]).convert(result_format, millis=is_falsy(exclude_millis))
            del self.TIMERS_DICTIONARY[timer_name]
            return delta

    def timer_restart(self, timer_name="Global", result_format="number", exclude_millis="True"):
        time_results = self.timer_stop(timer_name, result_format, exclude_millis)
        self.timer_start(timer_name)
        return time_results

    def timer_log(self, timer_name="Global", log_level="INFO", result_format="number", exclude_millis="True"):
        current_time = get_current_time_for_timers()
        if timer_name not in self.TIMERS_DICTIONARY:
            message = "Time '%s' is not started." % timer_name
            raise AssertionError(message)
        else:
            delta = Time(current_time - self.TIMERS_DICTIONARY[timer_name]).convert(result_format, millis=is_falsy(exclude_millis))
            msg = "Timer \"%s\" current results is %s" % (
                timer_name, Time(current_time - self.TIMERS_DICTIONARY[timer_name]).convert("verbose", millis=is_falsy(exclude_millis)))
            logger.write(msg, log_level)
            return delta

    def timer_should_be_lesser_then(self, expected_time, timer_name="Global"):
        if timer_name not in self.TIMERS_DICTIONARY:
            message = "Time '%s' is not started." % timer_name
            raise AssertionError(message)
        else:
            delta = get_current_time_for_timers() - self.TIMERS_DICTIONARY[timer_name]
            delta = Time._convert_time_to_seconds(self, delta)
            expected_time = Time._convert_time_to_seconds(self, expected_time)
            if delta > expected_time:
                message = "Timer '%s' is above expected time. Actual time %s > %s expected time " % (timer_name, delta, expected_time)
                raise AssertionError(message)
            return delta



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import io
import os
import time

import openpyxl
import unicodecsv as csv
from openpyxl import load_workbook
from robot.api import logger
from robot.libraries import DateTime
from robot.utils import asserts

from TestToolsMK.robot_instances import *


class ExcelKeywords(object):

    def __init__(self):
        self.wb = None  # type: openpyxl.workbook
        self.tb = None
        self.sheetNum = None
        self.sheetNames = None
        self.currentSheet = None  # type: openpyxl.worksheet
        self.fileName = None
        self.mode = None
        if os.name is "nt":
            self.tmpDir = "Temp"
        else:
            self.tmpDir = "tmp"

    def open_excel(self, filename, read_only=True, **kwargs):
        """
        Open spread excel and return spreadsheet names.
        Select first spreadsheet as current
        :param self: 
        :param filename: 
        :param kwargs:
        :return: 
        @param filename:
        @param read_only:
        """
        self.mode = read_only
        self.fileName = filename
        self.wb = load_workbook(self.fileName, read_only, **kwargs)
        self.sheetNames = self.wb.get_sheet_names()
        self.select_spreadsheet("")
        return self.sheetNames

    def save_working_excel(self, filename):
        """
        Save working file"
        """
        self.fileName = filename
        self.wb.save(self.fileName)
        self.open_excel(self.fileName, self.mode)

    def select_spreadsheet(self, name=""):

        if name == "":
            name = self.sheetNames[0]
            self.currentSheet = self.wb[name]
        else:
            self.currentSheet = self.wb[name]

    def get_all_values(self):

        table = []
        for row in self.currentSheet.rows:

            single_row = []
            for cell in row:
                single_row.append(cell.value)

            table.append(single_row)
        return table

    def get_cell_data_by_coordinates(self, column, row):
        """
        will use currently selected spreadsheet to change use method
        | select SpreadSheet | mySpreadSheet |
        | ${value}| get_cell_data_by_coordinates | A | 1 |

        """
        cell_value = self.currentSheet[column + row].value
        return cell_value

    def edit_data_by_coordinates(self, col_num, row_num, value=""):
        """
        Edit cell in open excel file by coordinates column and row number. Edit will use current spreadsheet
        | select SpreadSheet | mySpreadSheet |
        | ${old value}| edit_data_by_coordinates | 1 | 1 | Example |
        return old value
        """
        old = self.currentSheet.cell(row=int(row_num), column=int(col_num)).value
        self.currentSheet.cell(row=int(row_num), column=int(col_num)).value = value
        return old





#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
from PyPDF2.generic import NameObject

from TestToolsMK import robot_instances
import PyPDF2
from robot.api import logger


class PDFKeywords(object):
    pdfReader = None  # type: PyPDF2.PdfFileReader

    def __open_pdf(self, path):
        """
        read_pdf file read
        and return number of pages
        """
        pdfFileObj = open(path, 'rb')

        # creating a pdf reader object
        self.pdfReader = PyPDF2.PdfFileReader(pdfFileObj, strict=False)

        # closing the pdf file object

        # printing number of pages in pdf file
        # logger.info("Open PDF in " + path + " loaded with " + self.pdfReader.numPages + " pages")
        return self.pdfReader.numPages


    def get_file_pdf(self, path, split_by_page="False"):
        """
        Return text from file
        with splitByPage=True return list with splitted by page
        @return:
        """
        self.__open_pdf(path)

        pages = list()
        for pageNumber in range(0, self.pdfReader.numPages):
            # creating a page object
            pageObj = self.pdfReader.getPage(pageNumber)
            name = pageObj.extractText().strip().strip('\n')
            pages.append(name)
            # extracting text from page
            logger.debug("Page\t" + str(pageNumber) + " \tBody\t" + name)
        # self.pdfFileObj.close()

        if split_by_page != "False":
            return pages
        else:
            return "\n".join(pages)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski

import os
import os.path
import platform
import subprocess

try:
    # noinspection PyCompatibility
    from urlparse import urljoin
except ImportError:  # python3
    # noinspection PyCompatibility,PyUnresolvedReferences
    from urllib.parse import urljoin

import urllib
from robot.api import logger


class ImageMagickKeywords(object):
    def __init__(self):
        if os.path.isfile(self.get_convert_path):
            logger.debug("Convert file exits path, path used :" + self.get_convert_path)
        else:
            message = "Missing file convert.exe"
            logger.debug(message)

        if os.path.isfile(self.get_compare_path):
            logger.debug("Compare file exits , path used :" + self.get_compare_path)
        else:
            message = "Missing file compare.exe"
            logger.debug(message)

        if os.path.isfile(self.get_identify_path):
            logger.debug("Identify file exits, path used :" + self.get_identify_path)
        else:
            message = "Missing file identify.exe"
            logger.debug(message)

    @property
    def get_magick_home(self):
        try:
            if platform.system() == "Windows":
                return os.environ['MAGICK_HOME']
            if platform.system() == "Linux":
                return "/usr/bin/"

        except os.error as e:
            message = "Missing system variable 'MAGICK_HOME'" + e
            logger.warn(message)
            return message

    @property
    def get_compare_path(self):
        try:
            if platform.system() == "Windows":
                return os.path.normpath(self.get_magick_home + "\\" + "compare.exe")
            if platform.system() == "Linux":
                return os.path.normpath("/usr/bin/compare")
        except os.error as e:
            logger.warn("Missing file compare" + e)
            return "missing"

    @property
    def get_identify_path(self):
        try:
            if platform.system() == "Windows":
                return os.path.normpath(self.get_magick_home + "\\" + "identify.exe")
            if platform.system() == "Linux":
                return os.path.normpath("/usr/bin/identify")

        except os.error as e:
            logger.warn("Missing file identify" + e)
            return "missing"

    @property
    def get_convert_path(self):
        try:
            if platform.system() == "Windows":
                return os.path.normpath(self.get_magick_home + "\\" + "convert.exe")
            if platform.system() == "Linux":
                return os.path.normpath("/usr/bin/convert")

        except os.error as e:
            logger.warn("Missing file convert.exe" + e)
            return "missing"

    def image_self_check(self):
        if os.path.isfile(self.get_convert_path):
            logger.info("Convert file exits path, path used :" + self.get_convert_path)
        else:
            message = "Missing file convert.exe"
            raise AssertionError(message)

        if os.path.isfile(self.get_compare_path):
            logger.info("Compare file exits , path used :" + self.get_compare_path)
        else:
            message = "Missing file compare.exe"
            raise AssertionError(message)

        if os.path.isfile(self.get_identify_path):
            logger.info("Identify file exits, path used :" + self.get_identify_path)
        else:
            message = "Missing file identify.exe"
            raise AssertionError(message)

        argument_list = [self.get_convert_path, "--version"]
        try:
            procces = subprocess.Popen(argument_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # , shell=True
            procces.wait()
            output = procces.communicate()
            logger.info("returnCode \t:" + str(procces.returncode))
            if procces.returncode != 0:
                message = "processing failed msg= %s" % str(output)
                raise AssertionError(message)
            logger.info("stdout \t:" + str(output[0]))
            logger.info("stderr \t:" + str(output[1]))

        except OSError as e:
            logger.error(e)

    def _compare_image_files(self, file_1_path, file_2_path, gif_file_path=None, delta_file_path=None, metric="RMSE", embedded_gif=True, embedded_delta=False,
            force_resize=True):
        file_1_path_normalized = os.path.normpath(file_1_path)
        file_2_path_normalized = os.path.normpath(file_2_path)

        if force_resize:
            file_1_width, file_1_height = self._get_info_for_image(file_1_path_normalized)
            self._resize_file(file_2_path_normalized, file_1_width, file_1_height)
        if delta_file_path is not None:
            delta_file_path_normalized = os.path.normpath(delta_file_path)
        else:
            delta_file_path = os.path.dirname(file_1_path_normalized) + "\\" + os.path.splitext(os.path.basename(file_1_path_normalized))[0] + "_" + \
                              os.path.splitext(os.path.basename(file_2_path_normalized))[0] + "_delta.png"
            delta_file_path_normalized = os.path.normpath(delta_file_path)

        if gif_file_path is not None:
            gif_file_path_normalized = os.path.normpath(gif_file_path)
        else:
            gif_file_path = os.path.dirname(file_1_path_normalized) + "\\" + os.path.splitext(os.path.basename(file_1_path_normalized))[0] + "_" + \
                            os.path.splitext(os.path.basename(file_2_path_normalized))[0] + ".gif"
            gif_file_path_normalized = os.path.normpath(gif_file_path)
        if os.path.isfile(file_1_path_normalized) and os.path.isfile(file_2_path_normalized):
            argument_list = [self.get_compare_path, "-metric", metric, file_1_path_normalized, file_2_path_normalized, delta_file_path_normalized]
            process = subprocess.Popen(argument_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            process.wait()
            output = process.communicate()
            if "error" in str(output):
                message = "processing failed msg= %s" % str(output)
                raise AssertionError(message)

            delta_percent = output[1].split("(")[1].replace(")", "")

            if gif_file_path is not None:
                self.create_gif_from_three_files(gif_file_path_normalized, file_1_path_normalized, file_2_path_normalized, delta_file_path_normalized,
                    embedded=embedded_gif)
            if delta_file_path is None:
                todo = True
                # TODO maybe remove files
            else:
                if embedded_delta:
                    self._embed_screenshot(delta_file_path_normalized)

            return float(delta_percent) * float(100), delta_file_path_normalized, gif_file_path_normalized

    def compare_image_files(self, file_1_path, file_2_path, gif_file_path=None, delta_file_path=None, metric="RMSE", embedded_gif=True, embedded_delta=False,
            force_resize=True):
        results = self._compare_image_files(file_1_path, file_2_path, gif_file_path, delta_file_path, metric, embedded_gif, embedded_delta, force_resize)

        return results[0]

    def image_should_be_difference_less_then(self, file_1_path, file_2_path, difference_percent=1, gif_file_path=None, delta_file_path=None, embedded_gif=True,
            embedded_delta=False, force_resize=True):
        """difference_percent test to 0 mean both images are identical """
        results = self._compare_image_files(file_1_path, file_2_path, gif_file_path, delta_file_path, embedded_gif=embedded_gif, embedded_delta=embedded_delta,
            force_resize=force_resize)
        if float(results[0]) > float(difference_percent):
            message = "Difference between files is greater then expected actual %.2f > %.2f expected percent" % (float(results[0]), float(difference_percent))
            raise AssertionError(message)
        else:
            logger.info("Image check successful ")
        return results[0]

    def create_gif_from_three_files(self, gif_file_path, file_1_path, file_2_path, file_3_path, delay_ms=100, loop=0, embedded=True):
        files_list = [file_1_path, file_2_path, file_3_path]
        self.create_gif_from_list_of_files(gif_file_path, files_list, delay_ms, loop, embedded)

    def create_gif_from_list_of_files(self, gif_file_path, files_list_path, delay_ms=100, loop=0, embedded=True):
        files_list_path_normalized = []
        for singleFile in files_list_path:
            files_list_path_normalized.append(os.path.normpath(singleFile))
        gif_file_path_normalized = os.path.normpath(gif_file_path)
        if all(os.path.isfile(singleFile) == True for singleFile in files_list_path_normalized):
            argument_list = [self.get_convert_path, "-delay", str(delay_ms), "-loop", str(loop)]
            argument_list.extend(files_list_path_normalized)
            argument_list.append(gif_file_path_normalized)
            process = subprocess.Popen(argument_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            process.wait()
            if embedded:
                self._embed_screenshot(gif_file_path_normalized)

        else:
            message = "Gif Creation failed."
            for singleFile in files_list_path_normalized:
                if os.path.isfile(singleFile):
                    message += "File missing %s." % singleFile
            raise AssertionError(message)

    @staticmethod
    def _embed_screenshot(path, level="INFO", width="1200px"):
        link = urljoin('file:', urllib.pathname2url(os.path.normpath(path)))
        logger.write('</td></tr><tr><td colspan="3"><a href="%s"><img src="%s" width="%s"></a>' % (link, link, width), level, html=True)

    def _get_info_for_image(self, file_name):
        argument_list = [self.get_identify_path, "-quiet", "-format", "%[fx:w]\\n%[fx:h]", file_name]
        try:
            procces = subprocess.Popen(argument_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # , shell=True
            procces.wait()
            output = procces.communicate()
            if procces.returncode != 0:
                message = "processing failed msg= %s" % str(output)
                raise AssertionError(message)

            # logger.info("file Name \t:" + file_name+str(output))
            table_results = output[0].split()
            return table_results[0], table_results[1]
        except OSError as e:
            logger.error(e)

    def _resize_file(self, file_path_normalized, width, height):
        if os.path.isfile(file_path_normalized):
            argument_list = [self.get_convert_path, "-resize", width + "x" + height + "!", file_path_normalized, file_path_normalized]
            process = subprocess.Popen(argument_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            process.wait()
            output = process.communicate()
            if process.returncode != 0:
                message = "processing failed msg= %s" % str(output)
                raise AssertionError(message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski
import requests
from robot.api import logger


class SendNotificationKeywords(object):
    @staticmethod
    def send_notification_to_phone(msg, identifier):
        """
        To use its based on https://github.com/mashlol/notify
        link to android app (get identifier from this app)
        https://play.google.com/store/apps/details?id=com.kevinbedi.notify

        """
        params_to_send = {'to': identifier, 'text': msg}
        r = requests.get('https://us-central1-notify-b7652.cloudfunctions.net/sendNotification', params=params_to_send)

        logger.debug("Send Url:\t" + r.url)
        logger.debug("Response Url:\t" + r.text)
        if "\"success\":true" not in r.text:
            message = "Failed to send message err=%s" % r.text
            raise AssertionError(message)

        logger.info("Send Message \'" + msg + "\' to user with id " + identifier)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright (c) 2015 Cutting Edge QA Marcin Koperski


"""Setup script for Robot's Marcin Koperski bundle distributions"""

import os
import sys

from setuptools import setup

src_path = os.path.join(os.path.dirname(__file__), 'src')
sys.path.insert(0, src_path)

__version_file_path__ = os.path.join(src_path, 'TestToolsMK', 'VERSION')
__version__ = open(__version_file_path__, 'r').read().strip()

setup(
    name='robotframework-MarcinKoperski', version=__version__,
    description='RobotFramework Marcin Koperski bundle',
    author='Marcin Koperski', license='AGPL',
    author_email='marcin.koperski+pypi@gmail.com',
    url=u'https://github.com/IlfirinPL/robotframework-MarcinKoperski',
    download_url=u'https://github.com/IlfirinPL/robotframework-MarcinKoperski/archive/master.zip'.format(
        version=__version__), keywords=['robotframework', 'MarcinKoperski'],
    package_dir={u'': 'src'}, packages=['TestToolsMK'],
    exclude_package_data={'': ['.git', '.git/*', '.idea', '.gitignore', 'doc/*', 'atest/*']},
    package_data={'TestToolsMK': ['VERSION']},
    install_requires=[
        u'robotframework>=3.0',
        u'robotframework-databaselibrary',
        u'robotframework-imaplibrary',
        u'robotframework-seleniumlibrary',
        u'gspread',
        u'unicodecsv',
        u'oauth2client',
        u'robotframework-httplibrary',
        u'openpyxl'

    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Cutting Edge QA Marcin Koperski

import csv
import os

from robot.api import logger
from robot.libraries import DateTime

from TestToolsMK.robot_instances import validate_create_artifacts_dir, bi


class LoggerKeywords(object):

    @staticmethod
    def log_variable_to_file(name, comment="", output_file="Artifacts/variables.csv"):
        log_file = validate_create_artifacts_dir(output_file)
        logger.debug("Log to file " + log_file)
        fieldnames = ['Time', 'Test Case Name', 'Variable Name', 'Variable Value', 'Comment']
        current_time = DateTime.get_current_date(result_format="%Y.%m.%d %H:%M:%S")
        test_case_name = str(bi().get_variable_value("${TEST_NAME}"))
        suite_name = str(bi().get_variable_value("${SUITE_NAME}"))
        variable_value = name

        # TODO
        # get variable name is not working
        # variable_name = _Variables._get_var_name(bi(),str(name))
        # bi().get_variable_value("${" + variable_name + "}", "Missing!!!")

        with open(log_file, 'a') as csv_file:
            writer_csv = csv.writer(csv_file, dialect='excel')
            if os.stat(log_file).st_size < 10:
                writer_csv.writerow(fieldnames)
            writer_csv.writerow([current_time, suite_name + "." + test_case_name, name, variable_value, comment])

    # noinspection PyProtectedMember
    @staticmethod
    def set_log_level_none():
        log_level_history = bi().get_variable_value("${LOG_LEVEL_HISTORY}")
        if log_level_history is None:
            log_level_history = []
        old = bi().set_log_level("None")
        log_level_history.append(old)
        bi().set_global_variable("${LOG_LEVEL_HISTORY}", log_level_history)

    # noinspection PyProtectedMember
    @staticmethod
    def set_log_level_restore():
        log_level_history = bi().get_variable_value("${LOG_LEVEL_HISTORY}")
        if not log_level_history:
            bi().set_log_level('INFO')
        else:
            last = log_level_history.pop()
            bi().set_log_level(last)
